@@ -295,7 +295,7 @@ static struct button_widget button_widgets[] = {
     { 0,     0 },
     { GDK_r, 0 }},
 
-    {FN_ABSOLUTE_VALUE,     "abs",
+    {FN_ABSOLUTE_VALUE_FUNC, "abs",
     { 0,     0 },
     { GDK_u, 0 }},
 
@@ -445,7 +445,8 @@ static struct button_widget button_widgets[] = {
 };
 #define NBUTTONS (sizeof(button_widgets) / sizeof(struct button_widget))
 
-#define UI_FILE PACKAGE_UI_DIR "/gcalctool.ui"
+#define UI_FILE      PACKAGE_UI_DIR "/gcalctool.ui"
+#define UI_FINC_FILE PACKAGE_UI_DIR "/financial.ui"
 
 #define MAXBITS 64      /* Bit panel: number of bit fields. */
 
@@ -520,7 +521,10 @@ typedef struct {
     GtkWidget *gradian_radio;          /* Gradian radio button. */
 
     /* Programming mode widgets */
-    GtkWidget *base_radios[MAXBASES];
+    GtkWidget *binary_radio;
+    GtkWidget *octal_radio;
+    GtkWidget *decimal_radio;
+    GtkWidget *hexadecimal_radio;
     GtkWidget *word_length_radios[3];             /* Wordlength radio buttons. */
 
     GdkAtom clipboard_atom;
@@ -552,6 +556,29 @@ typedef enum {
     POPUP_CENTERED   /* Center popup within baseframe */
 } PopupLocation;
 
+static void load_ui(GtkBuilder *ui, const gchar *filename)
+{
+    GError *error = NULL;
+    GtkWidget *dialog;
+
+    gtk_builder_add_from_file(ui, filename, &error);
+    if (error == NULL)
+        return;
+        
+    dialog = gtk_message_dialog_new(NULL, 0,
+                                    GTK_MESSAGE_ERROR,
+                                    GTK_BUTTONS_NONE,
+                                    /* Translators: Title of the error dialog when unable to load the UI files */
+                                    N_("Error loading user interface"));
+    gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
+                                             /* Translators: Description in UI error dialog when unable to load the UI files. %s is replaced with the error message provided by GTK+ */
+                                             N_("A required file is missing or damaged, please check your installation.\n\n%s"), error->message);
+    gtk_dialog_add_buttons(GTK_DIALOG(dialog), GTK_STOCK_QUIT, GTK_RESPONSE_ACCEPT, NULL);
+    
+    gtk_dialog_run(GTK_DIALOG(dialog));
+    exit(0);
+}
+
 static void set_data(GtkBuilder *ui, const gchar *object_name, const gchar *name, gpointer value)
 {
     g_object_set_data(gtk_builder_get_object(ui, object_name), name, value);
@@ -673,13 +700,13 @@ ui_update_modifier_mode()
 {
     static char *sine_labels[]      = {
         /* Translators: The sine button */
-        N_("Sin"),
+        N_("sin"),
         /* Translators: The inverse sine button */        
-        N_("Sin<sup>-1</sup>"),
+        N_("sin<sup>−1</sup>"),
         /* Translators: The hyperbolic sine button */
-        N_("Sinh"),
+        N_("sinh"),
         /* Translators: The inverse hyperbolic sine button */
-        N_("Sinh<sup>-1</sup>")};
+        N_("sinh<sup>−1</sup>")};
     static char *sine_tooltips[]    = {
         /* Translators: The sine tooltip */
         N_("Sine [k]"),
@@ -692,13 +719,13 @@ ui_update_modifier_mode()
     static int  sine_functions[]    = {FN_SIN, FN_ASIN, FN_SINH, FN_ASINH};
     static char *cosine_labels[]    = {
         /* Translators: The cosine button */
-        N_("Cos"),
+        N_("cos"),
         /* Translators: The inverse cosine button */        
-        N_("Cos<sup>-1</sup>"),
+        N_("cos<sup>−1</sup>"),
         /* Translators: The hyperbolic cosine button */
-        N_("Cosh"),
+        N_("cosh"),
         /* Translators: The inverse hyperbolic cosine button */
-        N_("Cosh<sup>-1</sup>")};
+        N_("cosh<sup>−1</sup>")};
     static char *cosine_tooltips[]  = {
         /* Translators: The cosine tooltip */
         N_("Cosine [j]"),
@@ -711,13 +738,13 @@ ui_update_modifier_mode()
     static int  cosine_functions[]  = {FN_COS, FN_ACOS, FN_COSH, FN_ACOSH};
     static char *tangent_labels[]   = {
         /* Translators: The tangent button */
-        N_("Tan"),
+        N_("tan"),
         /* Translators: The inverse tangent button */        
-        N_("Tan<sup>-1</sup>"),
+        N_("tan<sup>−1</sup>"),
         /* Translators: The hyperbolic tangent button */
-        N_("Tanh"),
+        N_("tanh"),
         /* Translators: The inverse hyperbolic tangent button */
-        N_("Tanh<sup>-1</sup>")};
+        N_("tanh<sup>−1</sup>")};
     static char *tangent_tooltips[] = {
         /* Translators: The tangent tooltip */
         N_("Tangent [w]"),
@@ -731,36 +758,36 @@ ui_update_modifier_mode()
     
     static char *ln_labels[]        = {
         /* Translators: The natural logaritm button */
-        N_("Ln"),
+        N_("ln"),
         /* Translators: The e to the power of x button */
         N_("e<sup><i>x</i></sup>")};
     static char *ln_tooltips[]      = {
         /* Translators: Tooltip for the natural log button */
-        N_("Natural log [n]"),
+        N_("Natural logarithm [n]"),
         /* Translators: Tooltip for the e to the power of x button */
         N_("e to the power of the displayed value [N]")};
     static int ln_functions[]       = {FN_NATURAL_LOGARITHM, FN_E_POW_X};
     
     static char *log_labels[]       = {
         /* Translators: The 10-based logaritm button */
-        N_("Log"),
+        N_("log"),
         /* Translators: The 10 to the power of x button */
         N_("10<sup><i>x</i></sup>")};
     static char *log_tooltips[]     = {
         /* Translators: Tooltip for the log base 10 button */
-        N_("Base 10 log [g]"),
+        N_("Base 10 logarithm [g]"),
         /* Translators: Tooltip for the 10 to the power of x button */
         N_("10 to the power of displayed value [G]")};
     static int log_functions[]      = {FN_LOGARITHM, FN_10_POW_X};
     
     static char *log2_labels[]      = {
         /* Translators: The 2-based logaritm button */
-        N_("Log<sub>2</sub>"),
+        N_("log<sub>2</sub>"),
         /* Translators: The 2 to the power of x button */
         N_("2<sup><i>x</i></sup>")};
     static char *log2_tooltips[]    = {
         /* Translators: Tooltip for the log base 2 button */
-        N_("Base 2 log [h]"),
+        N_("Base 2 logarithm [h]"),
         /* Translators: Tooltip for the 2 to the power of x button */
         N_("2 to the power of the displayed value [H]")};
     static int log2_functions[]     = {FN_LOGARITHM2, FN_2_POW_X};
@@ -850,7 +877,7 @@ ui_set_trigonometric_mode(MPAngleUnit units)
 
 
 void
-ui_set_numeric_mode(BaseType mode)
+ui_set_numeric_mode(DisplayFormat mode)
 {
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(X.display_mode_radios[mode]), 1);
 }
@@ -939,19 +966,6 @@ ui_get_display(void)
 }
 
 
-static int
-get_cursor(void)
-{
-    gint pos;
-    g_object_get(G_OBJECT(X.display_buffer), "cursor-position", &pos, NULL);
-    
-    /* Convert the last position to -1 */
-    if (pos == gtk_text_buffer_get_char_count(X.display_buffer))
-        return -1;
-    else
-        return pos;
-}
-
 void
 ui_set_bitfield(int enabled, guint64 bits)
 {
@@ -971,7 +985,21 @@ ui_set_bitfield(int enabled, guint64 bits)
 
 static void do_button(int function, int arg)
 {
-    do_expression(function, arg, get_cursor());
+    GtkTextIter start, end;
+    gint cursor_start, cursor_end;
+
+    if(gtk_text_buffer_get_selection_bounds(X.display_buffer, &start, &end)) {
+        cursor_start = gtk_text_iter_get_offset(&start);
+        cursor_end = gtk_text_iter_get_offset(&end);
+    }
+    else {
+        g_object_get(G_OBJECT(X.display_buffer), "cursor-position", &cursor_start, NULL);
+        if (cursor_start == gtk_text_buffer_get_char_count(X.display_buffer))
+            cursor_start = -1;
+        cursor_end = cursor_start;
+    }
+
+    do_expression(function, arg, cursor_start, cursor_end);
 }
 
 static void
@@ -996,7 +1024,7 @@ ui_set_mode(ModeType mode)
 
         // FIXME: These should affect display but not the actual UI settings
         if (mode != PROGRAMMING)
-            ui_set_base(DEC);
+            ui_set_base(10);
         if (mode != SCIENTIFIC) {
             ui_set_numeric_mode(FIX);
             do_button(FN_SET_ACCURACY, DEFAULT_ACCURACY);
@@ -1174,7 +1202,7 @@ ui_set_display(char *str, int cursor)
     if (cursor < 0) {
         adj = gtk_scrolled_window_get_hadjustment(
                  GTK_SCROLLED_WINDOW(X.scrolledwindow));
-        gtk_adjustment_set_value(adj, adj->upper - adj->page_size);
+        gtk_adjustment_set_value(adj, gtk_adjustment_get_upper(adj) - gtk_adjustment_get_page_size(adj));
     }
 }
 
@@ -1238,15 +1266,33 @@ ui_beep()
 
 
 void
-ui_set_base(BaseType base)
+ui_set_base(int base)
 {
-    int i, baseval = basevals[(int) base];
+    GtkWidget *widget;
+    int i;
     
     v->base = base;
 
     for (i = 0; i < 16; i++)
-        gtk_widget_set_sensitive(X.digit_buttons[i], i < baseval);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(X.base_radios[base]), 1);
+        gtk_widget_set_sensitive(X.digit_buttons[i], i < base);
+    switch (base)
+    {
+    case 2:
+        widget = X.binary_radio;
+        break;
+    case 8:
+        widget = X.octal_radio;
+        break;
+    case 10:
+        widget = X.decimal_radio;
+        break;
+    case 16:
+        widget = X.hexadecimal_radio;
+        break;
+    default:
+        return;
+    }
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
 }
 
 
@@ -1285,8 +1331,8 @@ ui_set_registers_visible(gboolean visible)
     gtk_widget_realize(X.register_dialog);
 
     if (visible) {
-        if (gdk_window_is_visible(X.register_dialog->window)) {
-            gdk_window_raise(X.register_dialog->window);
+        if (gdk_window_is_visible(gtk_widget_get_window(X.register_dialog))) {
+            gdk_window_raise(gtk_widget_get_window(X.register_dialog));
             return;
         }
         position_popup(X.main_window, X.register_dialog, POPUP_ABOVE);
@@ -1467,9 +1513,9 @@ G_MODULE_EXPORT
 void
 base_cb(GtkWidget *widget)
 {
-    BaseType base;
+    int base;
 
-    base = (BaseType) g_object_get_data(G_OBJECT(widget), "base_mode");
+    base = (int) g_object_get_data(G_OBJECT(widget), "base_mode");
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
         do_button(FN_SET_BASE, base);
 }
@@ -1558,7 +1604,8 @@ exchange_menu_cb(GtkMenuItem *menu)
 
 G_MODULE_EXPORT
 void
-finc_activate_cb(GtkWidget *widget) {
+finc_activate_cb(GtkWidget *widget)
+{
     gint dialog, field;
 
     dialog = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "finc_dialog"));
@@ -1613,19 +1660,10 @@ finc_response_cb(GtkWidget *widget, gint response_id)
 static void
 setup_finc_dialogs(void)
 {
-    GError *error = NULL;
     int i, j;
     
-    // FIXME: Use same code as main UI
     X.financial = gtk_builder_new();
-    gtk_builder_add_from_file(X.financial, PACKAGE_UI_DIR "/financial.ui", &error);
-    if (error != NULL)
-    {
-        g_object_unref(X.financial);
-        X.financial = NULL;
-        g_warning("Error loading financial UI: %s\n", error->message);
-        return;
-    }
+    load_ui(X.financial, UI_FINC_FILE);
     
     set_int_data(X.financial, "ctrm_dialog", "finc_dialog", FINC_CTRM_DIALOG);
     set_int_data(X.financial, "ddb_dialog", "finc_dialog", FINC_DDB_DIALOG);
@@ -1657,7 +1695,7 @@ update_constants_menu(void)
     int i;
 
     for (i = 0; i < MAX_CONSTANTS; i++) {
-        display_make_number(&v->display, value, MAXLINE, constant_get_value(i), DEC, TRUE);
+        display_make_number(&v->display, value, MAXLINE, constant_get_value(i), 10, TRUE);
         SNPRINTF(mline, MAXLINE, 
                  "<span weight=\"bold\">%s_%1d:</span> %s [%s]", _("C"), i, 
                  value, 
@@ -1782,7 +1820,7 @@ create_constants_model()
     for (i = 0; i < MAX_CONSTANTS; i++) {
         gtk_list_store_append(model, &iter);
         
-        display_make_number(&v->display, constant, MAXLINE, constant_get_value(i), DEC, TRUE);
+        display_make_number(&v->display, constant, MAXLINE, constant_get_value(i), 10, TRUE);
         gtk_list_store_set(model, &iter,
                            COLUMN_NUMBER, i,
                            COLUMN_EDITABLE, TRUE,
@@ -1834,7 +1872,7 @@ ui_make_registers()            /* Calculate memory register frame values. */
         gtk_entry_set_text(GTK_ENTRY(X.register_entries[n]), mval);
 
         SNPRINTF(key, MAXLINE, "register%d", n);
-        display_make_number(&v->display, value, MAXLINE, &temp, DEC, TRUE);
+        display_make_number(&v->display, value, MAXLINE, &temp, 10, TRUE);
         set_resource(key, value);
     }
 }
@@ -1845,7 +1883,7 @@ save_win_position()
 {
     int x, y;
 
-    (void) gdk_window_get_origin(X.main_window->window, &x, &y);
+    (void) gdk_window_get_origin(gtk_widget_get_window(X.main_window), &x, &y);
     set_int_resource(R_XPOS, x);
     set_int_resource(R_YPOS, y);
 }
@@ -2031,7 +2069,7 @@ button_cb(GtkWidget *widget, GdkEventButton *event)
         update_memory_menus();
 
         if (event == NULL) {
-            gdk_window_get_origin(widget->window, &loc.x, &loc.y);
+            gdk_window_get_origin(gtk_widget_get_window(widget), &loc.x, &loc.y);
             loc.x += widget->allocation.x;
             loc.y += widget->allocation.y;
             gtk_menu_popup(GTK_MENU(menu), NULL, NULL, menu_pos_func,
@@ -2118,6 +2156,18 @@ main_window_key_press_cb(GtkWidget *widget, GdkEventKey *event)
         }
     }
     
+    /* Whitespace */
+    if (event->keyval == GDK_space && state == 0) {
+        do_button(FN_SPACE, 0);
+        return TRUE;
+    }
+    
+    /* Absolute value */
+    /*if (event->keyval == GDK_bar && state == 0) {
+        do_button(FN_ABSOLUTE_VALUE, 0);
+        return TRUE;
+    }*/
+    
     /* Delete in display */
     if (event->keyval == GDK_Delete && state == 0 && (event->state & GDK_SHIFT_MASK) == 0) {
         do_button(FN_DELETE, 0);
@@ -2159,7 +2209,7 @@ main_window_key_press_cb(GtkWidget *widget, GdkEventKey *event)
                     button_widgets[i].function)
                     do_button(button_widgets[i].function, 0);
                 else
-                   button_cb(button, NULL);
+                    button_cb(button, NULL);
                 return TRUE;
             }
         }
@@ -2474,28 +2524,9 @@ create_main_window()
     GtkSizeGroup *size_group;
     GtkAccelGroup *accel_group;
     GtkWidget *treeview;
-    GError *error = NULL;
    
     X.ui = gtk_builder_new();
-    gtk_builder_add_from_file(X.ui, UI_FILE, &error);
-    if (error != NULL) {
-        GtkWidget *dialog;
-        
-        dialog = gtk_message_dialog_new(NULL, 0,
-                                        GTK_MESSAGE_ERROR,
-                                        GTK_BUTTONS_NONE,
-                                        /* Translators: Title of the error dialog when unable to load the UI files */
-                                        N_("Error loading user interface"));
-        // FIXME: Use error->message
-        g_warning("Unable to load file %s: %s\n", UI_FILE, error->message);
-        gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
-                                                 /* Translators: Description in UI error dialog when unable to load the UI files. %s is replaced with the path of the missing file */
-                                                 N_("The user interface file %s is missing or unable to be loaded. Please check your installation."), UI_FILE);
-        gtk_dialog_add_buttons(GTK_DIALOG(dialog), GTK_STOCK_QUIT, GTK_RESPONSE_ACCEPT, NULL);
-        
-        gtk_dialog_run(GTK_DIALOG(dialog));
-        exit(0);
-    }
+    load_ui(X.ui, UI_FILE);
     gtk_builder_connect_signals(X.ui, NULL);
 
     X.clipboard_atom   = gdk_atom_intern("CLIPBOARD", FALSE);
@@ -2524,10 +2555,10 @@ create_main_window()
     X.degree_radio     = GET_WIDGET("degrees_radio");
     X.gradian_radio    = GET_WIDGET("gradians_radio");
     X.radian_radio     = GET_WIDGET("radians_radio");
-    X.base_radios[0]   = GET_WIDGET("binary_radio");
-    X.base_radios[1]   = GET_WIDGET("octal_radio");
-    X.base_radios[2]   = GET_WIDGET("decimal_radio");
-    X.base_radios[3]   = GET_WIDGET("hexadecimal_radio");
+    X.binary_radio     = GET_WIDGET("binary_radio");
+    X.octal_radio      = GET_WIDGET("octal_radio");
+    X.decimal_radio    = GET_WIDGET("decimal_radio");
+    X.hexadecimal_radio      = GET_WIDGET("hexadecimal_radio");
     X.display_mode_radios[0] = GET_WIDGET("engineering_radio");
     X.display_mode_radios[1] = GET_WIDGET("fixed_point_radio");
     X.display_mode_radios[2] = GET_WIDGET("scientific_radio");
@@ -2673,7 +2704,7 @@ create_main_window()
 
     X.display_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(X.display_item));
     gtk_widget_ensure_style(X.display_item);
-    font_desc = pango_font_description_copy(X.display_item->style->font_desc);
+    font_desc = pango_font_description_copy(gtk_widget_get_style(X.display_item)->font_desc);
     pango_font_description_set_size(font_desc, 16 * PANGO_SCALE);
     gtk_widget_modify_font(X.display_item, font_desc);
     pango_font_description_free(font_desc);
@@ -2684,14 +2715,16 @@ create_main_window()
     gtk_widget_realize(X.main_window);
     set_win_position();
 
-    g_object_set_data(G_OBJECT(X.radian_radio), "trig_mode", GINT_TO_POINTER(MP_RADIANS));
-    g_object_set_data(G_OBJECT(X.degree_radio), "trig_mode", GINT_TO_POINTER(MP_DEGREES));
-    g_object_set_data(G_OBJECT(X.gradian_radio), "trig_mode", GINT_TO_POINTER(MP_GRADIANS));
-    for (i = 0; i < 4; i++)
-        g_object_set_data(G_OBJECT(X.base_radios[i]), "base_mode", GINT_TO_POINTER(i));
-    for (i = 0; i < 3; i++)
-        g_object_set_data(G_OBJECT(X.display_mode_radios[i]), "numeric_mode", GINT_TO_POINTER(i));
-    
+    g_object_set_data(G_OBJECT(X.radian_radio),          "trig_mode", GINT_TO_POINTER(MP_RADIANS));
+    g_object_set_data(G_OBJECT(X.degree_radio),          "trig_mode", GINT_TO_POINTER(MP_DEGREES));
+    g_object_set_data(G_OBJECT(X.gradian_radio),         "trig_mode", GINT_TO_POINTER(MP_GRADIANS));
+    g_object_set_data(G_OBJECT(X.binary_radio),          "base_mode", GINT_TO_POINTER(2));
+    g_object_set_data(G_OBJECT(X.octal_radio),           "base_mode", GINT_TO_POINTER(8));
+    g_object_set_data(G_OBJECT(X.decimal_radio),         "base_mode", GINT_TO_POINTER(10));
+    g_object_set_data(G_OBJECT(X.hexadecimal_radio),     "base_mode", GINT_TO_POINTER(16));
+    g_object_set_data(G_OBJECT(X.display_mode_radios[0]), "numeric_mode", GINT_TO_POINTER(ENG));
+    g_object_set_data(G_OBJECT(X.display_mode_radios[1]), "numeric_mode", GINT_TO_POINTER(FIX));
+    g_object_set_data(G_OBJECT(X.display_mode_radios[2]), "numeric_mode", GINT_TO_POINTER(SCI));
     g_object_set_data(G_OBJECT(X.word_length_radios[0]), "wordlen_mode", GINT_TO_POINTER(64));
     g_object_set_data(G_OBJECT(X.word_length_radios[1]), "wordlen_mode", GINT_TO_POINTER(32));
     g_object_set_data(G_OBJECT(X.word_length_radios[2]), "wordlen_mode", GINT_TO_POINTER(16));
@@ -2740,7 +2773,6 @@ void
 ui_init(int *argc, char ***argv)
 {  
     gchar *path;
-    const gchar *home;
     int value;
 
     gtk_init(argc, argv);
@@ -2754,8 +2786,7 @@ ui_init(int *argc, char ***argv)
     else
         X.mode = BASIC;
 
-    home = g_get_home_dir();
-    path = g_build_path(home, RCNAME, NULL);
+    path = g_build_filename(g_get_home_dir(), RCNAME, NULL);
     gtk_rc_parse(path);
     g_free(path);
 
@@ -2804,12 +2835,6 @@ ui_start(void)
     ui_set_wordlen(v->wordlen);
     ui_set_trigonometric_mode(v->ttype);
     ui_set_numeric_mode(v->display.format);
-
-    /* Focus on the clear button */
-    if (X.mode == BASIC)
-        gtk_widget_grab_focus(GTK_WIDGET(X.clear_buttons[0]));
-    else
-        gtk_widget_grab_focus(GTK_WIDGET(X.clear_buttons[1]));
     
     gtk_widget_show(X.main_window);
 
